C-----------------------------------------------------------------------
c
c     This file is part of the Test Set for BVP solvers
c     http://www.dm.uniba.it/~testset/
c
c        generic bvp_m-2 driver
c              bvp_m-2 is called bvp_m in the Test Set
c
c     DISCLAIMER: see
c     http://www.dm.uniba.it/~testset/disclaimer.php
c
c     The most recent version of this source file can be found at
c     http://www.dm.uniba.it/~testset/src/drivers/bvp_m.f
c
c
c-----------------------------------------------------------------------
       MODULE DEFINE_FCN



       CONTAINS

c!ODEs
       SUBROUTINE FSUB(X,Y,F)
		DOUBLE PRECISION ::X,Y(*),F(*)
		integer :: node, ipar(10)
		double precision :: rpar(10)
        common /neqnval/ node
	    common /rparipar/ rpar, ipar

        call feval(node,x,y(1:node),f(1:node),rpar,ipar)
c          write(*,*) 'f',node,x, f(1:node)

       END SUBROUTINE FSUB

c!Boundary conditions
      SUBROUTINE BCSUB(YA,YB,BCA,BCB)
	  DOUBLE PRECISION :: YA(*),YB(*),BCA(*),BCB(*)
      integer :: node, ipar(10)
      double precision :: rpar(10)
	  integer :: i, nlbc
      common /nlbc/ nlbc
      common /neqnval/ node
      common /rparipar/ rpar, ipar


       do  i=1,nlbc
          call bceval(i,node,ya,bca(i),rpar,ipar)
       end do
       do  i=nlbc+1,node
          call bceval(i,node,yb,bcb(i-nlbc),rpar,ipar)
       end do
       END SUBROUTINE BCSUB


         SUBROUTINE DFSUB(X,Y,DF)
        ! subroutine used to fill in for missing DFSUB in call to MIRKDC
        ! module. Has same argument list as DFSUB.
        !-------------------------------------------------------------------------
        !     CALLED BY: BVP_SOLVER
        !-------------------------------------------------------------------------
        !   Input arguments:
         integer :: node, ipar(10)
         double precision :: rpar(10)

	     common /rparipar/ rpar, ipar

         common /neqnval/ node
         DOUBLE PRECISION :: X,Y(1:node)

        !   Output argument:
         DOUBLE PRECISION :: DF(1:node,1:node)

        !-------------------------------------------------------------------------
           call jeval(node,x,y,df,rpar,ipar)


       END SUBROUTINE DFSUB


       SUBROUTINE DBCSUB(YA,YB,DYA,DYB)
        ! Dummy subroutine used to fill in for missing DGSUB in call to MIRKDC
        ! module. Has same argument list as DGSUB.
        !-------------------------------------------------------------------------
        !     CALLED BY: BVP_SOLVER
        !-------------------------------------------------------------------------
        !   Input arguments:
        integer :: node, ipar(10)
        double precision :: rpar(10)
	    common /rparipar/ rpar, ipar
        common /neqnval/ node
        integer i, nlbc
        common /nlbc/ nlbc
        DOUBLE PRECISION :: YA(1:node),YB(1:node)

        !   Output arguments:
        DOUBLE PRECISION :: DYA(1:nlbc,*),DYB(1:node-nlbc,*)
        !-------------------------------------------------------------------------



         do  i=1,nlbc
          call dbceval(i,node,ya,dya(i,1:node),rpar,ipar)

         end do
         do  i=nlbc+1,node
          call dbceval(i,node,yb,dyb(i-nlbc,1:node),rpar,ipar)
         end do


        END SUBROUTINE DBCSUB


c!Initial guess
        SUBROUTINE GUESS_Y(X,Y)
        DOUBLE PRECISION :: X,Y(*)
        PARAMETER(NUDIM=20,NXXDIM=10000)
c!c nudim must be equal ti nudima
c!c nxxdim must be equal to nxxdim defined in the main file
        integer nudima,nmguess,neqn,mstar
        common /nudimcom/ nudima,nmguess
        common /inguess/ xguess,yguess,dmguess
        common /neqnval/ neqn
        double precision xguess(nxxdim), yguess(nudim,nxxdim)
         double precision xx(1)

       xx(1)=x
       call interp(neqn, 1, xx, 1, Y, nudim,
     +                    nmguess, xguess, yguess)

       END SUBROUTINE GUESS_Y


c ===================================================================================

       subroutine interp(ncomp, nmsh, xx, nudim, u,nuold,
     &                    nmold, xxold, uold)

      implicit double precision (a-h, o-z)
      dimension xx(*), u(nudim,*), xxold(*), uold(nudim,*)

!* blas: dcopy

      parameter (zero = 0.0d+0)

!*  interp performs piecewise linear interpolation of the old
!*  solution uold at the nmold old mesh points xxold onto the nmsh
!*  new mesh points xx, producing an interpolated solution u.
!*  Note that no assumption is made that the new mesh has
!*  more points than the old, nor that the new and old mesh
!*  points are related in a specific way (except that their first
!*  and last points are identical).

!*  By construction, xx(1) = xxold(1).  Copy the first ncomp
!*  components of uold into those of u.

      call dcopy(ncomp, uold(1,1), 1, u(1,1), 1)

      i = 2
      do 100 im = 2, nmsh-1

   50    continue
         if (i .gt. nmold) return

!*  Check whether the im-th point in the new mesh lies strictly
!*  to the right of, or to the left of (or exactly on) the
!*  i-th point in the old mesh.


         if (xx(im) .gt. xxold(i)) then
            i = i + 1
            go to 50
         else
            xdif = xxold(i) - xx(im)
            if (xdif .eq. zero) then

!*  xx(im) and xxold(i) are identical.

               call dcopy(ncomp, uold(1,i), 1, u(1,im), 1)
               i = i + 1
            else
               xint = xxold(i) - xxold(i-1)
               xrat = xdif/xint
               do 70 k = 1, ncomp
                  u(k,im) = uold(k,i) + xrat*(uold(k,i-1)-uold(k,i))
   70          continue
            endif
         endif

  100 continue
      call dcopy(ncomp, uold(1,nmold), 1, u(1,nmsh), 1)
      return
      end subroutine interp

!c ===================================================================================


      END MODULE DEFINE_FCN


       PROGRAM DRIVERBVP_M

       USE DEFINE_FCN
       USE BVP_M

      IMPLICIT NONE


      TYPE(BVP_SOL) :: SOL !BVP_SOLVER main solution structure


      INTEGER   IPRINT, IND,
     +        NTOL,  NLBC, NMSH, NFXPNT, NMAX,
     +        IFLBVP, INDNMS, K, NMINIT, IDUM


      INTEGER TRACE
      INTEGER, PARAMETER :: nxxdim=10000
      INTEGER, PARAMETER :: nudim = 20
      INTEGER :: NUGDIM, NSPDIM
      PARAMETER(NUGDIM=NUDIM)
      PARAMETER(NSPDIM=NUDIM)
      DOUBLE PRECISION,ALLOCATABLE :: u(:,:), xx(:)
      DOUBLE PRECISION :: xguess(1:nxxdim),yguess(1:nudim,1:nxxdim)
      DOUBLE PRECISION:: dmguess(1:nudim,1:nxxdim)
      DOUBLE PRECISION,ALLOCATABLE :: UACC(:,:),YREF(:,:)
      DOUBLE PRECISION,ALLOCATABLE :: tol(:),yval0(:),atol(:)
      INTEGER, ALLOCATABLE :: LTOL(:)
      INTEGER :: ms(nudim)

      DOUBLE PRECISION :: RPAR(10)
      INTEGER :: IPAR(10)
      DOUBLE PRECISION :: ALEFT, ARIGHT
      DOUBLE PRECISION :: ckappa,ckappa1,ckappa2,gamma1,sigma
      DOUBLE PRECISION :: rtol,  pi, lambda
      INTEGER ::   iprob, j
      INTEGER ::  nmguess
      LOGICAL :: givmsh,   givey, numjac, numbcjac

      DOUBLE PRECISION :: XPT, HMIN, HMAX, ERRMAX,XPTOLD, EXX,  HINT
      LOGICAL :: linear, full



      DOUBLE PRECISION precis(3)
      integer n0, maxmesh
      LOGICAL  GIVEU
      logical  useC
      logical  ACCMESHFAIL
      LOGICAL  use_c, comp_c, tolvec


      EXTERNAL feval
      EXTERNAL jeval
      EXTERNAL bceval
      EXTERNAL dbceval

      INTEGER lnblnk
      EXTERNAL lnblnk

c this common need to be defined in order to run TWPBVPC successfully
c give information about some parameters

      character fullnm*25, problm*15, bvptype*5
      character driver*9, solver*8

      parameter (driver = 'BVP_MD', solver='BVP_M')

      real gettim, timtim, cputim
      external gettim
      double precision scd, mescd

      character  fileout*140,  filepath*100
      character  namefile*100
      logical printsolout, solref
      integer nindsol, indsol(nudim),neqn, mstar,i, kstep
      logical  stab_kappa
      common /nlbc/ nlbc
      common /neqnval/ neqn
      common /rparipar/ rpar, ipar
      common /inguess/ xguess,yguess,dmguess
      integer iset(6)

c! Variables for BVP_SOLVER

c       ! counter for solution output
       INTEGER :: METH
c       ! Method to be employed by BVP_SOLVER.
       INTEGER :: ERRORCON
c       !error control method
       DOUBLE PRECISION :: ERREST
c       ! Estimate of global error.
       DOUBLE PRECISION :: TTOL
c       !tolerance
c!DOUBLE PRECISION :: ALEFT=0D0,ARIGHT=1D0 ! Define interval using global


c-----------------------------------------------------------------------
c     check the problem definition interface date
c-----------------------------------------------------------------------


      call chkdat(driver,solver,20130531)

c-----------------------------------------------------------------------
c     get the problem dependent parameters
c-----------------------------------------------------------------------
      fullnm(1:25)=' '
      fileout(1:140)=' '
      filepath(1:100)=' '
      namefile(1:100)=' '
      bvptype(1:5)=' '
      problm(1:15)=' '


      call prob(fullnm,problm,bvptype,
     +          neqn,aleft,aright,nlbc,ms,
     +          numjac,numbcjac,linear,rpar,ipar)


      mstar=0
      do i=1,neqn
        mstar=mstar+ms(i)
      end do

      if (  mstar .ne. neqn  )then
         print *, 'BVP_MD: ERROR: ',
     +            'BVP_MD cannot solve higher order problems'
         stop
      endif

      if ( bvptype(1:lnblnk(bvptype)).ne.'BVP' .and.
     +       bvptype(1:lnblnk(bvptype)) .ne. 'SPBVP' )then
         print *, 'BVP_MD: ERROR: ',
     +            'unknown Test Set problem bvptype ', bvptype
         stop
      endif


      ALLOCATE(u(neqn,nxxdim))
      ALLOCATE(xx(nxxdim))
      ALLOCATE(YREF(neqn,NXXDIM))

      ALLOCATE(UACC(neqn,NXXDIM))
      ALLOCATE(tol(neqn),yval0(neqn),atol(neqn))
      ALLOCATE(LTOL(neqn))



c-----------------------------------------------------------------------
c     read the tolerances and initial stepsize
c-----------------------------------------------------------------------

      call getinp(driver,problm,solver,fullnm,
     +            tolvec,tol,atol,bvptype)

c-----------------------------------------------------------------------
c     get the initial values
c-----------------------------------------------------------------------



      call init(neqn,aleft,aright,ms,yval0,nmsh,givmsh,giveu,xguess,
     +     yguess,dmguess,neqn)
      n0 = nmsh

      nmguess = nmsh
      XX(1:NMSH)=XGUESS(1:NMSH)
      XX(1) = aleft
      XX(NMSH) = aright
      U(1:neqn,1:nmsh) = yguess(1:neqn,1:nmsh)


      call settolerances(neqn,tol,atol,tolvec,ntol,ltol)
      rtol = tol(1)
      if (ntol .ne. neqn)  then
         print *, 'BVP_MD: ERROR: ',
     +            'bvp_md require atol of size', neqn
         stop
      end if

      call  setoutput(neqn,solref,printsolout,
     +                        nindsol,indsol)

      call getout(printsolout)

      !Set method
      METH=6
!Set tolerance
      TTOL=TOL(1)
!
!Set ERROR_CONTROL_METHOD
      ERRORCON = 1
!for defect control
      TRACE = -1
! full trace
      SOL = BVP_INIT(neqn,nlbc,xx(1:nmsh),
     +      GUESS_Y,MAX_NUM_SUBINTERVALS=nxxdim)




      if (printsolout) then

          call getarg(1,filepath)
          call getarg(2,namefile)

          if (lnblnk(namefile) .gt. 0) then
            write(fileout,'(a,a,a,a,a)') filepath(1:lnblnk(filepath)),
     +     namefile(1:lnblnk(namefile)),'_',
     +     solver(1:lnblnk(solver)),'.txt'


             open(UNIT=90,FILE=fileout)

             call mfileout(namefile,solver,filepath,nindsol,indsol)

          else
            write(fileout,'(a,a,a,a,a)') filepath(1:lnblnk(filepath)),
     +      problm(1:lnblnk(problm)),'_',
     +      solver(1:lnblnk(solver)),'.txt'



             open(UNIT=90,FILE=fileout)

             call mfileout(problm,solver,filepath,nindsol,indsol)
          end if

      end if

c-----------------------------------------------------------------------
c     call of the subroutine BVP_M
c-----------------------------------------------------------------------


      timtim = gettim()
      timtim = gettim() - timtim

      cputim = gettim()




      if (bvptype(1:3).eq.'BVP' .or.
     +            bvptype(1:5).eq.'SPBVP') then

       if (numjac .and. .not. numbcjac) then
            SOL = BVP_SOLVER(SOL,FSUB,
     + BCSUB,TRACE=TRACE, DBCDY=DBCSUB,STOP_ON_FAIL=.TRUE.,
     + METHOD=METH,ERROR_CONTROL=ERRORCON,TOL=TTOL,HOERROR=ERREST,
     + COND = ckappa,ICOUNT=ISET)

      elseif (numjac .and. numbcjac) then
          SOL = BVP_SOLVER(SOL,FSUB,
     + BCSUB,TRACE=TRACE,STOP_ON_FAIL=.TRUE.,
     + METHOD=METH,ERROR_CONTROL=ERRORCON,TOL=TTOL,HOERROR=ERREST,
     + COND = ckappa,ICOUNT=ISET)
      elseif ( .not. numjac .and.  numbcjac) then
            SOL = BVP_SOLVER(SOL,FSUB,
     + BCSUB,TRACE=TRACE, DFDY=DFSUB,STOP_ON_FAIL=.TRUE.,
     + METHOD=METH,ERROR_CONTROL=ERRORCON,TOL=TTOL,HOERROR=ERREST,
     + COND = ckappa, ICOUNT=ISET)


      else

      SOL = BVP_SOLVER(SOL,FSUB,
     + BCSUB,TRACE=TRACE, DFDY=DFSUB, DBCDY=DBCSUB,
     + STOP_ON_FAIL=.TRUE.,
     + METHOD=METH,ERROR_CONTROL=ERRORCON,TOL=TTOL,HOERROR=ERREST,
     + COND = ckappa,ICOUNT=ISET)
      end if

      else
      print *, 'BVP_MD: ERROR: ',
     +            'unknown Test Set problem bvptype ', bvptype
         stop

      end if
        cputim = gettim() - cputim - timtim
        iflbvp= SOL%info
         if (iflbvp.gt.0) then
            print *, 'BVP_M: ERROR: ',
     +               'BVP_M returned IFLBVP = ', IFLBVP
            stop
         endif

       nmsh = SOL%NPTS
       xx(1:nmsh) = SOL%X(1:nmsh)
       U(1:neqn,1:nmsh) = sol%Y(1:neqn,1:nmsh)



c-----------------------------------------------------------------------
c     print numerical solution in endpoint and integration statistics
c-----------------------------------------------------------------------
c
c



      if (solref) then

        call solut(neqn,xx(1:nmsh),yref(1:neqn,1:nmsh),nmsh,rpar,ipar)

        call getscd(mescd,scd,neqn,xx(1:nmsh),yref(1:neqn,1:nmsh),
     +     u(1:neqn,1:nmsh),
     +    problm, tolvec,tol,atol, nmsh,printsolout,
     +    fileout,nindsol,indsol)


        call report(
     +   driver,problm,solver,
     +   tol,tol,
     +   iset,cputim,scd,mescd,nmsh,
     +   ckappa,ckappa1,ckappa2,gamma1,sigma
     +)
      else
c add the computation of a reference solution in the mesh point


         CALL APPREFSOL(neqn,nmsh,aleft,aright,
     +            xx,u,uacc,yval0,rtol,ntol,atol,
     +            linear,
     +            accmeshfail, sol, meth, ttol,errorcon,
     +            nudim,nxxdim,numjac,numbcjac)

         if (.not. accmeshfail) THEN
            call getscd(mescd,scd,neqn,xx(1:nmsh),uacc(1:neqn,1:nmsh),
     +     u(1:neqn,1:nmsh),
     +    problm, tolvec,tol,atol, nmsh,printsolout,
     +    fileout,nindsol,indsol)


         call report(
     +   driver,problm,solver,
     +   tol,tol,
     +   iset,cputim,scd,mescd,nmsh,
     +   ckappa,ckappa1,ckappa2,gamma1,sigma
     +)

         ELSE
            call printsol(neqn,nmsh,xx(1:nmsh),u(1:neqn,1:nmsh),
     +   problm,fileout,nindsol,indsol,printsolout)

         end if
      end if


      if (printsolout) then
         close(90)
      end if


      DEALLOCATE(u)
      DEALLOCATE(xx)
c      DEALLOCATE(xguess,yguess,dmguess)
      DEALLOCATE(YREF)
      DEALLOCATE(UACC)
      DEALLOCATE(tol,yval0,atol)


       CALL BVP_TERMINATE(SOL)

                
      END PROGRAM DRIVERBVP_M




c
c----------------------------------------------------------------------
c Computation of the reference solution
c
c
      SUBROUTINE APPREFSOL(neqn,nmsh,aleft,aright,
     +            xx,u,uacc,yval0,rtol,ntol,atol,
     +            linear,
     +            accmeshfail, sol, meth, ttol,errorcon,
     +            nudim,nxxdim,numjac,numbcjac)


       USE DEFINE_FCN
       USE BVP_M

      IMPLICIT NONE
       INTEGER METH, ERRORCON
        DOUBLE PRECISION TTOL

        TYPE(BVP_SOL) :: SOL !BVP_SOLVER main solution structure
        TYPE(BVP_SOL) :: SOLACC !BVP_SOLVER main solution structure

      INTEGER NUDIM, NXXDIM, LISERIES, IPRINT, IND,IWRK,
     +         NTOL, IPAR, LTOL, NLBC, NMSH, NFXPNT, NMAX,
     +         I,J, NMSHACC, K,  NMINIT, IDUM,  IFLBVP_A
      DOUBLE PRECISION  TOL,  WRK, RPAR, ALEFT, ARIGHT,  XX, U,
     +                 CKAPPA, FIXPNTACC, XXACC, UACC,YACC
      DOUBLE PRECISION CKAPPA_A,CKAPPA1_A,GAMMA1_A,SIGMA_A, CKAPPA2_A
      LOGICAL numjac,numbcjac


      DIMENSION  rpar(10),ipar(10)
      DIMENSION U(Neqn,NXXDIM), XX(NXXDIM)
      DIMENSION UACC(Neqn,NXXDIM), XXACC(NXXDIM)
      DIMENSION YACC(Neqn,NXXDIM)
      DOUBLE PRECISION XGUESS(NXXDIM),YGUESS(Neqn,NXXDIM), RTOL
      DIMENSION TOL(Neqn),   FIXPNTACC(NXXDIM)
      DOUBLE PRECISION ATOL(Neqn)
      DOUBLE PRECISION SPLC(neqn,nxxdim+15), yval0(neqn)

      DOUBLE PRECISION SPLC_a(neqn,nxxdim+15)
      INTEGER   indnms_a,nmguess, neqn,kstep,nspdim, kstep_a
      INTEGER    iset_a(7),  int_options(6)
      DOUBLE PRECISION  precis(3)
      integer TRACE
      LOGICAL   LINEAR, GIVEU, GIVMSH
      logical   useC



        DOUBLE PRECISION :: ERREST
      logical full, ACCMESHFAIL
      logical  stab_kappa
      LOGICAL  use_c, comp_c, PRINTSOL
      common /rparipar/ rpar, ipar
      common /nlbc/ nlbc
      integer ncomp
      common /neqnval/ ncomp


      precis(3) = epsilon(1.0d0)
      precis(2) = huge(1.0d0)
      precis(1) = tiny(1.0d0)



       TRACE=-1
c full TRACE=2
      NMSHACC= 2*NMSH-1


      IF (NMSHACC .gt. NXXDIM ) THEN
          ACCMESHFAIL = .true.
         RETURN
      ENDIF


      DO I=2,NMSH-1
        FIXPNTACC(2*(I-1)-1)=(XX(I-1)+XX(I))/2.0D0
        FIXPNTACC(2*(I-1))=XX(I)
      END DO
      FIXPNTACC(2*(NMSH-2)+1)=(XX(NMSH-1)+XX(NMSH))/2.0
      NFXPNT= 2*(NMSH-2)+1



      Xguess(1) = XX(1)
      DO I=2,2*(NMSH-2)+2
         Xguess(I) =FIXPNTACC(I-1)
      END DO
      Xguess(NMSHACC) = XX(NMSH)
c      nmshacc=nmsh
c      Xguess(1:nmshacc) = XX(1:nmshacc)
c      Xguess(NMSHACC) = XX(NMSH)

      XXACC(1:NMSHACC)=XGUESS(1:NMSHACC)
      XXACC(1) = aleft
      XXACC(NMSHACC) = aright
      nmguess=nmshacc

c      YGUESS(1:neqn,1:nmguess)=U(1:neqn,1:nmsh)
      givmsh = .TRUE.
      giveu = .TRUE.

      call  BVP_EVAL(SOL,XGUESS(1:nmshacc),YGUESS(1:neqn,1:nmshacc))

      YACC(1:neqn,1:nmshacc) = yguess(1:neqn,1:nmshacc)



      TTOL = TTOL/10.0d0

      SOLACC = BVP_INIT(neqn,nlbc,xguess(1:nmshacc),
     +      GUESS_Y,MAX_NUM_SUBINTERVALS=nxxdim)


      accmeshfail = .false.

       if (numjac .and. .not. numbcjac) then
            SOLACC = BVP_SOLVER(SOLACC,FSUB,
     + BCSUB,TRACE=TRACE, DBCDY=DBCSUB,
     + METHOD=METH,ERROR_CONTROL=ERRORCON,TOL=TTOL,HOERROR=ERREST)
      elseif (numjac .and. numbcjac) then
          SOLACC = BVP_SOLVER(SOLACC,FSUB,
     + BCSUB,TRACE=TRACE,
     + METHOD=METH,ERROR_CONTROL=ERRORCON,TOL=TTOL,HOERROR=ERREST)
      elseif ( .not. numjac .and.  numbcjac) then
            SOLACC = BVP_SOLVER(SOLACC,FSUB,
     + BCSUB,TRACE=TRACE, DFDY=DFSUB,
     + METHOD=METH,ERROR_CONTROL=ERRORCON,TOL=TTOL,HOERROR=ERREST)

      else


      SOLACC = BVP_SOLVER(SOLACC,FSUB,
     + BCSUB,TRACE=TRACE, DFDY=DFSUB, DBCDY=DBCSUB,
     + METHOD=METH,ERROR_CONTROL=ERRORCON,TOL=TTOL,HOERROR=ERREST)
      end if


       iflbvp_a= SOLACC%info




      IF (IFLBVP_A .gt. 0 ) THEN
         ACCMESHFAIL = .true.
         write(*,*) 'accmeshfail 1', accmeshfail, stab_kappa
         RETURN
      ENDIF


       nmshacc = SOLACC%NPTS
       xxacc(1:nmshacc) = SOLACC%X(1:nmshacc)
       YACC(1:neqn,1:nmshacc) = solACC%Y(1:neqn,1:nmshacc)

      IF (NMSHACC .gt. NXXDIM ) THEN
         ACCMESHFAIL = .true.
          write(*,*) 'accmeshfail 2', accmeshfail
         RETURN
      ENDIF

      IF (NMSHACC .eq. NMSH) THEN
            ACCMESHFAIL = .true.
          write(*,*) 'accmeshfail 3', accmeshfail
         RETURN
      ENDIF

      IF (NMSHACC .eq. 0 .or. IFLBVP_A .GT. 0) THEN
         ACCMESHFAIL = .true.
         write(*,*) 'accmeshfail 4', accmeshfail
         RETURN
      ENDIF

      call  BVP_EVAL(SOLACC,XX(1:nmsh),UACC(1:ncomp,1:nmsh))

      UACC(1:neqn,1)=YACC(1:neqn,1)
      UACC(1:neqn,nmsh)=YACC(1:neqn,nmshacc)


      RETURN
      END

